import * as cdk from 'aws-cdk-lib';
import { ICfnConditionExpression, ICfnRuleConditionExpression } from 'aws-cdk-lib';
import { EvaluationContext, IEvaluationContext } from './context';
import { AnnotationsContext } from '../error-handling';
import { SubFragment } from '../parser/private/sub';
import { BooleanLiteral, GetPropIntrinsic, IntrinsicExpression, LazyLogicalId, NumberLiteral, RefIntrinsic, StringLiteral } from '../parser/template';
import { CdkConstruct, CdkObject, CfnResource as CfnResourceNode } from '../type-resolution';
import { InstanceMethodCallExpression, StaticMethodCallExpression } from '../type-resolution/callables';
import { TypedTemplateExpression } from '../type-resolution/expression';
import { DateLiteral } from '../type-resolution/literals';
export declare abstract class BaseEvaluator<T extends IEvaluationContext> {
    readonly context: T;
    constructor(context: T);
    abstract evaluateDescription(ctx: AnnotationsContext): any;
    abstract evaluateTemplateFormatVersion(ctx: AnnotationsContext): any;
    abstract evaluateMappings(ctx: AnnotationsContext): any;
    abstract evaluateParameters(ctx: AnnotationsContext): any;
    abstract evaluateResources(ctx: AnnotationsContext): any;
    abstract evaluateOutputs(ctx: AnnotationsContext): any;
    abstract evaluateTransform(ctx: AnnotationsContext): any;
    abstract evaluateMetadata(ctx: AnnotationsContext): any;
    abstract evaluateRules(ctx: AnnotationsContext): any;
    abstract evaluateHooks(ctx: AnnotationsContext): any;
    abstract evaluateConditions(ctx: AnnotationsContext): any;
    protected abstract evaluateNull(): any;
    protected abstract evaluateVoid(): any;
    protected abstract evaluateDate(x: DateLiteral): any;
    protected abstract evaluatePrimitive(x: StringLiteral | NumberLiteral | BooleanLiteral): any;
    protected abstract evaluateCfnResource(x: CfnResourceNode, ctx: AnnotationsContext): any;
    protected abstract evaluateConstruct(x: CdkConstruct, ctx: AnnotationsContext): any;
    protected abstract evaluateCdkObject(x: CdkObject, ctx: AnnotationsContext): any;
    protected abstract evaluateObject(xs: Record<string, TypedTemplateExpression>, ctx: AnnotationsContext): Record<string, unknown>;
    protected abstract evaluateArray(xs: TypedTemplateExpression[], ctx: AnnotationsContext): any;
    protected abstract evaluateCall(call: StaticMethodCallExpression | InstanceMethodCallExpression, ctx: AnnotationsContext): any;
    protected abstract evaluateEnum(fqn: string, choice: string): any;
    protected abstract evaluateInitializer(fqn: string, parameters: unknown[]): any;
    protected abstract fnBase64(x: string): any;
    protected abstract fnCidr(ipBlock: string, count: number, sizeMask?: string): any;
    protected abstract fnFindInMap(mapName: string, topLevelKey: string, secondLevelKey: string): any;
    protected abstract fnGetProp(logicalId: string, prop: string): any;
    protected abstract fnGetAtt(logicalId: string, attribute: string): any;
    protected abstract fnGetAzs(region: string): any;
    protected abstract fnIf(conditionName: string, ifYes: TypedTemplateExpression, ifNo: TypedTemplateExpression, ctx: AnnotationsContext): any;
    protected abstract fnImportValue(exportName: string): any;
    protected abstract fnJoin(separator: string, array: any[]): any;
    protected abstract fnRef(logicalId: string): any;
    protected abstract fnSelect(index: number, elements: any[]): any;
    protected abstract fnSplit(separator: string, value: string): any;
    protected abstract fnSub(fragments: SubFragment[], additionalContext: Record<string, any>): any;
    protected abstract fnTransform(transformName: string, parameters: Record<string, unknown>): any;
    protected abstract fnAnd(operands: unknown[]): any;
    protected abstract fnOr(operands: unknown[]): any;
    protected abstract fnNot(operand: unknown): any;
    protected abstract fnEquals(value1: unknown, value2: unknown): any;
    protected abstract fnLazyLogicalId(x: LazyLogicalId): any;
    protected abstract fnLength(x: unknown): any;
    protected abstract fnToJsonString(x: unknown): any;
    evaluateTemplate(ctx: AnnotationsContext): any;
    evaluate(x: TypedTemplateExpression, ctx: AnnotationsContext): any;
    protected evaluateIntrinsic(x: IntrinsicExpression, ctx: AnnotationsContext): any;
    protected resolveReference(intrinsic: RefIntrinsic | GetPropIntrinsic, ctx: AnnotationsContext): any;
}
export declare class Evaluator extends BaseEvaluator<EvaluationContext> {
    evaluateTemplate(ctx: AnnotationsContext): void;
    evaluateDescription(_ctx: AnnotationsContext): void;
    evaluateTemplateFormatVersion(_ctx: AnnotationsContext): void;
    evaluateMappings(ctx: AnnotationsContext): void;
    evaluateParameters(ctx: AnnotationsContext): void;
    evaluateResources(ctx: AnnotationsContext): void;
    private _evaluateResource;
    private _referenceForResourceLike;
    evaluateOutputs(ctx: AnnotationsContext): void;
    evaluateTransform(ctx: AnnotationsContext): void;
    evaluateMetadata(ctx: AnnotationsContext): void;
    evaluateRules(ctx: AnnotationsContext): void;
    evaluateHooks(ctx: AnnotationsContext): void;
    evaluateConditions(ctx: AnnotationsContext): void;
    protected evaluateNull(): undefined;
    protected evaluateVoid(): undefined;
    protected evaluatePrimitive(x: StringLiteral | NumberLiteral | BooleanLiteral): string | number | boolean;
    protected evaluateDate(x: DateLiteral): Date;
    protected evaluateCfnResource(x: CfnResourceNode, ctx: AnnotationsContext): cdk.CfnResource;
    protected evaluateConstruct(x: CdkConstruct, ctx: AnnotationsContext): any;
    protected evaluateCdkObject(x: CdkObject, ctx: AnnotationsContext): any;
    protected evaluateObject(xs: Record<string, TypedTemplateExpression>, ctx: AnnotationsContext): Record<string, unknown>;
    protected evaluateArray(xs: TypedTemplateExpression[], ctx: AnnotationsContext): any[];
    protected evaluateCall(call: StaticMethodCallExpression | InstanceMethodCallExpression, ctx: AnnotationsContext): any;
    protected evaluateEnum(fqn: string, choice: string): any;
    private _invokeInstanceMethod;
    private _invokeStaticMethod;
    protected evaluateInitializer(fqn: string, parameters: unknown[]): any;
    protected fnBase64(x: string): string;
    protected fnCidr(ipBlock: string, count: number, sizeMask?: string): string[];
    protected fnFindInMap(mapName: string, topLevelKey: string, secondLevelKey: string): string;
    protected fnGetProp(logicalId: string, prop: string): unknown;
    protected fnGetAtt(logicalId: string, attribute: string): string;
    protected fnGetAzs(region: string): string[];
    protected fnIf(conditionName: string, ifYes: TypedTemplateExpression, ifNo: TypedTemplateExpression, ctx: AnnotationsContext): cdk.ICfnRuleConditionExpression;
    protected fnImportValue(exportName: string): string;
    protected fnJoin(separator: string, array: any[]): string;
    protected fnRef(logicalId: string): string;
    protected fnSelect(index: number, elements: any[]): string;
    protected fnSplit(separator: string, value: string): string[];
    protected fnSub(fragments: SubFragment[], additionalContext: Record<string, any>): string;
    protected fnTransform(transformName: string, parameters: Record<string, unknown>): cdk.IResolvable;
    protected fnAnd(operands: ICfnConditionExpression[]): ICfnConditionExpression;
    protected fnOr(operands: ICfnConditionExpression[]): ICfnConditionExpression;
    protected fnNot(operand: ICfnConditionExpression): ICfnRuleConditionExpression;
    protected fnEquals(value1: unknown, value2: unknown): ICfnConditionExpression;
    protected fnLazyLogicalId(x: LazyLogicalId): string;
    protected fnLength(x: unknown): number;
    protected fnToJsonString(x: unknown): string;
    private _applyTags;
    private _applyDependsOn;
    private _applyOverrides;
}
