import { SubFragment } from '../private/sub';
export type TemplateExpression = StringLiteral | ObjectLiteral | ArrayLiteral | NumberLiteral | BooleanLiteral | NullLiteral | IntrinsicExpression;
export interface StringLiteral {
    readonly type: 'string';
    readonly value: string;
}
export interface NumberLiteral {
    readonly type: 'number';
    readonly value: number;
}
export interface BooleanLiteral {
    readonly type: 'boolean';
    readonly value: boolean;
}
export interface NullLiteral {
    readonly type: 'null';
}
export interface ObjectExpression<T> {
    readonly type: 'object';
    readonly fields: Record<string, T>;
}
export interface ArrayExpression<T> {
    readonly type: 'array';
    readonly array: T[];
}
export interface ArrayLiteral extends ArrayExpression<TemplateExpression> {
}
export interface ObjectLiteral extends ObjectExpression<TemplateExpression> {
}
export type IntrinsicExpression = UserIntrinsicExpression | LazyLogicalId;
export type UserIntrinsicExpression = RefIntrinsic | GetAttIntrinsic | GetPropIntrinsic | Base64Intrinsic | CidrIntrinsic | FindInMapIntrinsic | GetAZsIntrinsic | IfIntrinsic | ImportValueIntrinsic | JoinIntrinsic | SelectIntrinsic | SplitIntrinsic | SubIntrinsic | TransformIntrinsic | AndIntrinsic | OrIntrinsic | NotIntrinsic | EqualsIntrinsic | ArgsIntrinsic | LengthIntrinsic | ToJsonStringIntrinsic;
export interface RefIntrinsic {
    readonly type: 'intrinsic';
    readonly fn: 'ref';
    readonly logicalId: string;
}
export interface GetAttIntrinsic {
    readonly type: 'intrinsic';
    readonly fn: 'getAtt';
    readonly logicalId: string;
    readonly attribute: TemplateExpression;
}
export interface GetPropIntrinsic {
    readonly type: 'intrinsic';
    readonly fn: 'getProp';
    readonly logicalId: string;
    readonly property: string;
}
export interface Base64Intrinsic {
    readonly type: 'intrinsic';
    readonly fn: 'base64';
    readonly expression: TemplateExpression;
}
export interface CidrIntrinsic {
    readonly type: 'intrinsic';
    readonly fn: 'cidr';
    readonly ipBlock: TemplateExpression;
    readonly count: TemplateExpression;
    readonly netMask?: TemplateExpression;
}
export interface FindInMapIntrinsic {
    readonly type: 'intrinsic';
    readonly fn: 'findInMap';
    readonly mappingName: TemplateExpression;
    readonly key1: TemplateExpression;
    readonly key2: TemplateExpression;
}
export interface GetAZsIntrinsic {
    readonly type: 'intrinsic';
    readonly fn: 'getAzs';
    readonly region: TemplateExpression;
}
export interface IfIntrinsic {
    readonly type: 'intrinsic';
    readonly fn: 'if';
    readonly conditionName: string;
    readonly then: TemplateExpression;
    readonly else: TemplateExpression;
}
export interface ImportValueIntrinsic {
    readonly type: 'intrinsic';
    readonly fn: 'importValue';
    readonly export: TemplateExpression;
}
export interface JoinIntrinsic {
    readonly type: 'intrinsic';
    readonly fn: 'join';
    readonly separator: string;
    readonly list: ListIntrinsic;
}
export interface SelectIntrinsic {
    readonly type: 'intrinsic';
    readonly fn: 'select';
    readonly index: TemplateExpression;
    readonly objects: ListIntrinsic;
}
export interface SplitIntrinsic {
    readonly type: 'intrinsic';
    readonly fn: 'split';
    readonly separator: string;
    readonly value: TemplateExpression;
}
export interface SubIntrinsic {
    readonly type: 'intrinsic';
    readonly fn: 'sub';
    readonly fragments: SubFragment[];
    readonly additionalContext: Record<string, TemplateExpression>;
}
export interface TransformIntrinsic {
    readonly type: 'intrinsic';
    readonly fn: 'transform';
    readonly transformName: string;
    readonly parameters: Record<string, TemplateExpression>;
}
export interface AndIntrinsic {
    readonly type: 'intrinsic';
    readonly fn: 'and';
    readonly operands: TemplateExpression[];
}
export interface OrIntrinsic {
    readonly type: 'intrinsic';
    readonly fn: 'or';
    readonly operands: TemplateExpression[];
}
export interface NotIntrinsic {
    readonly type: 'intrinsic';
    readonly fn: 'not';
    readonly operand: TemplateExpression;
}
export interface EqualsIntrinsic {
    readonly type: 'intrinsic';
    readonly fn: 'equals';
    readonly value1: TemplateExpression;
    readonly value2: TemplateExpression;
}
export interface ArgsIntrinsic {
    readonly type: 'intrinsic';
    readonly fn: 'args';
    readonly array: TemplateExpression[];
}
export interface LazyLogicalId {
    readonly type: 'intrinsic';
    readonly fn: 'lazyLogicalId';
    readonly value?: string;
    readonly errorMessage?: string;
}
export interface LengthIntrinsic {
    readonly type: 'intrinsic';
    readonly fn: 'length';
    readonly list: ListIntrinsic;
}
export interface ToJsonStringIntrinsic {
    readonly type: 'intrinsic';
    readonly fn: 'toJsonString';
    readonly value: Record<string, TemplateExpression>;
}
export declare const INTRINSIC_NAME_MAP: Record<Exclude<IntrinsicExpression['fn'], 'lazyLogicalId'>, string>;
export declare function isExpression(x: unknown): x is TemplateExpression;
export declare function assertExpression(x: unknown): TemplateExpression;
export declare function parseExpression(x: unknown): TemplateExpression;
export declare function parseObject(x: unknown): {
    [k: string]: TemplateExpression;
};
type ListIntrinsic = ArrayLiteral | RefIntrinsic | CidrIntrinsic | FindInMapIntrinsic | GetAttIntrinsic | GetAZsIntrinsic | IfIntrinsic | SplitIntrinsic;
export declare function ifField<A extends object, K extends keyof A, B>(xs: A, k: K, fn: (x: NonNullable<A[K]>) => B): B | undefined;
export declare function assertIntrinsic<T extends IntrinsicExpression['fn']>(x: TemplateExpression, fns: T[]): IntrinsicExpression & {
    fn: T;
};
/**
 * Wraps an expression inside an ArrayLiteral. If it is already an ArrayLiteral,
 * return the expression unmodified.
 */
export declare function asArrayLiteral(x: TemplateExpression): ArrayLiteral;
export {};
